import sys
import random
import logging
import asyncio
import concurrent
import os

from virgotools import *

log_to_cfg(level=logging.INFO)

DATA = ''

 ## input parsing ##
if len(sys.argv) in (2, 3):
    cfg_path = os.path.abspath(sys.argv[1])  # do this first, FdIO changes working dir
    cfg_dir, cfg_file = os.path.split(cfg_path)

async def run_worker(loop, worker):
    no_data_counter = 0
    worker_counter = 0
    log.info('Starting run_worker')
    global DATA, query_period
    #executor = concurrent.futures.ProcessPoolExecutor(max_workers=1)
    executor = concurrent.futures.ThreadPoolExecutor(max_workers=1)
    while not fd.CfgFinished():
        await asyncio.sleep(query_period)
        try:
            future = loop.run_in_executor(executor, worker)
            result = await asyncio.wait_for(future, 60, loop=loop)
            if isinstance(result,tuple):
                (data, DATA) = result
            else:
                data = DATA = result
            #DATA = data = await loop.run_in_executor(executor, worker)
            #log.info('data is %s', data)
        except asyncio.TimeoutError:
            fd.CfgReachState(fd.CfgServerError)
            fd.CfgMsgAddUserInfo("Timeout from worker process!")
            fd.CfgMsgSendWithTimeout(0.001)
            data = ""
            log.warn("Timeout from worker process!")
            fd.CfgReachState(fd.CfgServerError)
            cm.WriteSmsString(data)
            worker_counter += 1
            if worker_counter > 100:
                log.warn("Worker problem persisting, exiting")
                sys.exit('Worker problem persisting, exiting')
        except Exception as e:
            fd.CfgMsgAddError(str(e))
        else:
            worker_counter = 0
            #log.info('data is %s', data)
            if data == "":
                no_data_counter += 1
                if no_data_counter > 5:
                    fd.CfgReachState(fd.CfgServerError)
                    cm.WriteSmsString(data)
                else:
                    #no WriteSmsString, keep previous sample
                    fd.CfgReachState(fd.CfgServerActive)
            else:
                no_data_counter = 0
                fd.CfgReachState(fd.CfgServerGolden)
                cm.WriteSmsString(data)
    log.warn('Worker finished')
    #os.killpg(0, signal.SIGKILL)
    sys.exit(0)


async def manage_communication():
    global p
    log.info('Starting manage_communication')
    #os.setpgrp()
    counter = 0
    while not fd.CfgFinished():
        #execute CfgMsgAddUserInfo not at loop speed to avoid possible flood of errors
        counter += 1
        if counter == 90:
            fd.CfgMsgAddUserInfo("GPS:%i - %s" % (now_gps(), DATA))
            #fd.CfgMsgAddUserInfo("GPS: %i" % (now_gps()))
            counter = 0
        fd.CfgMsgSendWithTimeout(0.001)
        await asyncio.sleep(0.01)
    log.warn('Manage_communication finished')
    #os.killpg(0, signal.SIGKILL)
    if p != None:
        p.kill()
    sys.exit(0)

def run_server(worker, *, init=None, cleanup=None):
    global query_period, p
    log.info('Initialization')
    fdio = FdIO(sys.argv)
    fd.CfgReachState(fd.CfgServerIdle)
    query_period = int(parse_key(cfg_path, 'QUERY_PERIOD')[0][0])
    if init is not None:
        p = init()
    cm.CmMessageInstallHandler(cm.H_FbGetAllSmsData,"FbGetAllSmsData")
    fd.CfgReachState(fd.CfgServerConfigured)

    loop = asyncio.get_event_loop()
    task1 = loop.create_task(run_worker(loop, worker))
    task2 = loop.create_task(manage_communication())

    #Store the list of pending tasks
    tasks_pending = [task1,task2]
    #asyncio.ensure_future(run_worker(loop, worker))
    #asyncio.ensure_future(manage_communication())
    fd.CfgReachState(fd.CfgServerActive)
    try:
        loop.run_forever()
    finally:
        if cleanup is not None:
            p = cleanup()
        #Cancel the pending tasks
        for t in tasks_pending:
            t.cancel()
        #Run the loop until all the task are complete
        while not all([t.done() for t in tasks_pending]):
            loop._run_once()
        #Now close the loop
        loop.close()
        #os.killpg(0, signal.SIGKILL)
        sys.exit(0)

if __name__ == '__main__':
    main()

